#include <vtkActor.h>
#include <vtkVersion.h>
#include <vtkCellArray.h>
#include <vtkCellData.h>
#include <vtkQuad.h>
#include <vtkPolyData.h>
#include <vtkPolyDataMapper.h>
#include <vtkSmartPointer.h>
#include <vtkRenderWindow.h>
#include <vtkRenderer.h>
#include <vtkRenderWindowInteractor.h>

int main(int , char *[])
{
  // Add the points to a vtkPoints object
  vtkSmartPointer<vtkPoints> points = vtkSmartPointer<vtkPoints>::New();
  vtkSmartPointer<vtkCellArray> quads = vtkSmartPointer<vtkCellArray>::New();

  int iMax = 5;
  int jMax = 5;
  for (int j = 0; j < jMax; ++j)
  {
    for (int i = 0; i < iMax; ++i)
    {
      double p[3] = { i, j, 0.0 };
      points->InsertNextPoint(p);
    }
  }

  for (int j = 0; j < jMax-1; ++j)
  {
    for (int i = 0; i < iMax-1; ++i)
    {
      // Create four point ids (must be in counter clockwise order)
      vtkSmartPointer<vtkQuad> quad = vtkSmartPointer<vtkQuad>::New();
      quad->GetPointIds()->SetId(0, iMax*(j) + i);
      quad->GetPointIds()->SetId(1, iMax*(j) + i + 1);
      quad->GetPointIds()->SetId(2, iMax*(j+1) + i + 1);
      quad->GetPointIds()->SetId(3, iMax*(j+1) + i);

      // Create a cell array to store the quad in
      quads->InsertNextCell(quad);
    }
  }

  // Create a polydata to store everything in
  vtkSmartPointer<vtkPolyData> polydata =
    vtkSmartPointer<vtkPolyData>::New();

  // Setup colors
  unsigned char red[3] = { 255, 0, 0 };
  unsigned char green[3] = { 0, 255, 0 };
  unsigned char blue[3] = { 0, 0, 255 };

  vtkSmartPointer<vtkUnsignedCharArray> colors =
    vtkSmartPointer<vtkUnsignedCharArray>::New();
  colors->SetNumberOfComponents(3);
  colors->SetName("Colors");

  for (int j = 0; j < jMax-1; ++j)
  {
    for (int i = 0; i < iMax-1; ++i)
    {
      vtkIdType cellId = (iMax-1)*j + i;

      if ((i+j)%2 == 0)
      {
        colors->InsertNextTupleValue(red);
      }
      else
      {
        colors->InsertNextTupleValue(green);
      }
    }
  }
  polydata->GetCellData()->SetScalars(colors);

  // Add the points and quads to the dataset
  polydata->SetPoints(points);
  polydata->SetPolys(quads);

  // Setup actor and mapper
  vtkSmartPointer<vtkPolyDataMapper> mapper =
    vtkSmartPointer<vtkPolyDataMapper>::New();
#if VTK_MAJOR_VERSION <= 5
  mapper->SetInput(polydata);
#else
  mapper->SetInputData(polydata);
#endif
  
  vtkSmartPointer<vtkActor> actor =
    vtkSmartPointer<vtkActor>::New();
  actor->SetMapper(mapper);
  
  // Setup render window, renderer, and interactor
  vtkSmartPointer<vtkRenderer> renderer =
    vtkSmartPointer<vtkRenderer>::New();
  vtkSmartPointer<vtkRenderWindow> renderWindow =
    vtkSmartPointer<vtkRenderWindow>::New();
  renderWindow->AddRenderer(renderer);
  vtkSmartPointer<vtkRenderWindowInteractor> renderWindowInteractor = 
    vtkSmartPointer<vtkRenderWindowInteractor>::New();
  renderWindowInteractor->SetRenderWindow(renderWindow);
  renderer->AddActor(actor);
  renderWindow->Render();
  renderWindowInteractor->Start();

  return EXIT_SUCCESS;
}
